#!/bin/sh
# Mount storage

# e.g. SCRIPT_PARAM_CONTAINER_ENV_VARS='{"SESSION_ID":"AAAA0000BBBB1111CCCC2222DDDD3333EEEE","SESSION_TOKEN":"abc123","SESSION_CALLBACK":"https://.../api/environment/registration/./_callback","GIT_REPO_URL":"https://github.com/vsls-contrib/guestbook"}'
echo "Set script to fail if command in script returns non zero return code"
set -eu pipefall

start_operation()
{
  ARG1=${2:-'na'}
  if [ "$ARG1" = "start" ] 
  then
    rm /.vsonline/vsoagent/bin/mountresult.json || true
    echo '[' >>  /.vsonline/vsoagent/bin/mountresult.json
  fi
    echo '{' >>  /.vsonline/vsoagent/bin/mountresult.json
	echo '"name":"'$1'",' >>  /.vsonline/vsoagent/bin/mountresult.json
	echo '"startTime":"'$(date -u +"%s")'",' >>  /.vsonline/vsoagent/bin/mountresult.json
}

end_operation()
{
   ARG1=${1:-'na'}
   echo '"endTime":"'$(date -u +"%s")'",' >>  /.vsonline/vsoagent/bin/mountresult.json
   echo '"succeeded":"true"' >>  /.vsonline/vsoagent/bin/mountresult.json
  if [ "$ARG1" = "end" ]
  then
   echo '}' >>  /.vsonline/vsoagent/bin/mountresult.json
   echo ']' >>  /.vsonline/vsoagent/bin/mountresult.json
  else
   echo '},' >>  /.vsonline/vsoagent/bin/mountresult.json
  fi
}

SCRIPT_PARAM_STORAGE='__Replace_Storage_Params__'

[ -z "$SCRIPT_PARAM_STORAGE" ] && echo "SCRIPT_PARAM_STORAGE parameter not set." && exit 1;

CLOUDENVSTORAGE_ACCOUNT=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; print json.load(sys.stdin)['storageAccountName']")
CLOUDENVSTORAGE_PASS=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; print json.load(sys.stdin)['storageAccountKey']")
CLOUDENVSTORAGE_SAS_TOKEN=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; payload = json.load(sys.stdin); result = '' if 'storageAccountSasToken' not in payload else payload['storageAccountSasToken']; print result")
CLOUDENVSTORAGE_SHARE=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; print json.load(sys.stdin)['storageShareName']")
CLOUDENVSTORAGE_SHARE_MOUNT_PATH=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; print json.load(sys.stdin)['storageShareMountPath']")
CLOUDENVSTORAGE_CONTAINER_LOGS_PATH=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; print json.load(sys.stdin)['storageContainerLogsPath']")
CLOUDENVSTORAGE_CONTAINER_DB_PATH=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; print json.load(sys.stdin)['storageContainerDbPath']")
CLOUDENVSTORAGE_FILE=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; print json.load(sys.stdin)['storageFileName']")
CLOUDENVSTORAGEARCHIVE_SAS_TOKEN=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; payload = json.load(sys.stdin); result = '' if 'storageArchiveBlobFileSasToken' not in payload else payload['storageArchiveBlobFileSasToken']; print result")
CLOUDENVSTORAGEARCHIVE_TARGET_SIZE=$(echo $SCRIPT_PARAM_STORAGE | python -c "import sys, json; payload = json.load(sys.stdin); result = '' if 'storageArchiveBlobTargetSizeGb' not in payload else payload['storageArchiveBlobTargetSizeGb']; print result")

[ -z "$CLOUDENVSTORAGE_ACCOUNT" ] && echo "CLOUDENVSTORAGE_ACCOUNT parameter not set." && exit 1;
[ -z "$CLOUDENVSTORAGE_PASS" ] && echo "CLOUDENVSTORAGE_PASS parameter not set." && exit 1;
[ -z "$CLOUDENVSTORAGE_SHARE" ] && echo "CLOUDENVSTORAGE_SHARE parameter not set." && exit 1;
[ -z "$CLOUDENVSTORAGE_SHARE_MOUNT_PATH" ] && echo "CLOUDENVSTORAGE_SHARE_MOUNT_PATH parameter not set." && exit 1;
[ -z "$CLOUDENVSTORAGE_CONTAINER_LOGS_PATH" ] && echo "CLOUDENVSTORAGE_CONTAINER_LOGS_PATH parameter not set." && exit 1;
[ -z "$CLOUDENVSTORAGE_CONTAINER_DB_PATH" ] && echo "CLOUDENVSTORAGE_CONTAINER_DB_PATH parameter not set." && exit 1;
[ -z "$CLOUDENVSTORAGE_FILE" ] && echo "CLOUDENVSTORAGE_FILE parameter not set." && exit 1;

echo "store credentials ..."
mkdir -p /etc/smbcredentials
printf "username=$CLOUDENVSTORAGE_ACCOUNT\npassword=$CLOUDENVSTORAGE_PASS\n" > /etc/smbcredentials/vsonlinestorage.cred
chmod 600 /etc/smbcredentials/vsonlinestorage.cred

echo "stop docker ..."
start_operation "mount:stopDocker" "start"
# Stop current running docker daemon
systemctl stop docker
end_operation

echo "mount file share ..."
start_operation "mount:mountAzureFileShare"
# Actual mount of the File Share here
mkdir -p $CLOUDENVSTORAGE_SHARE_MOUNT_PATH
mount -t cifs //$CLOUDENVSTORAGE_ACCOUNT.file.core.windows.net/$CLOUDENVSTORAGE_SHARE $CLOUDENVSTORAGE_SHARE_MOUNT_PATH -o vers=3.0,credentials=/etc/smbcredentials/vsonlinestorage.cred,dir_mode=0777,file_mode=0777,serverino
end_operation

echo "create a fileshare directory to store container logs..."
mkdir -p $CLOUDENVSTORAGE_CONTAINER_LOGS_PATH
chmod o+rw $CLOUDENVSTORAGE_CONTAINER_LOGS_PATH

echo "create a fileshare directory to store container db..."
mkdir -p $CLOUDENVSTORAGE_CONTAINER_DB_PATH
chmod o+rw $CLOUDENVSTORAGE_CONTAINER_DB_PATH

if [ ! -z "$CLOUDENVSTORAGEARCHIVE_SAS_TOKEN" ] 
then
    echo "delete /mnt/cloudenvdata/dockerlib so we have a clean start ..."
    rm $CLOUDENVSTORAGE_SHARE_MOUNT_PATH/$CLOUDENVSTORAGE_FILE

    echo "downloading archive blob to override /mnt/cloudenvdata/dockerlib ..."
    start_operation "restore:downloadblob"
    /.vsonline/azcopy cp "$CLOUDENVSTORAGEARCHIVE_SAS_TOKEN" "$CLOUDENVSTORAGE_SHARE_MOUNT_PATH/$CLOUDENVSTORAGE_FILE" --block-size-mb 100
    end_operation
    
    echo "expanding archive blob at /mnt/cloudenvdata/dockerlib ..."
    start_operation "restore:expandblob"
    resize2fs -p $CLOUDENVSTORAGE_SHARE_MOUNT_PATH/$CLOUDENVSTORAGE_FILE ${CLOUDENVSTORAGEARCHIVE_TARGET_SIZE}G
    end_operation
else
    start_operation "restore:notneeded"
    end_operation
fi

echo "mount file share to /var/lib/docker ..."
start_operation "mount:mountShareToDocker"
losetup -Pf $CLOUDENVSTORAGE_SHARE_MOUNT_PATH/$CLOUDENVSTORAGE_FILE
losetup
LOOP_DEVICE=`losetup | grep "$CLOUDENVSTORAGE_SHARE_MOUNT_PATH/$CLOUDENVSTORAGE_FILE" -m 1| cut -d" " -f1`
echo "LOOP_DEVICE : $LOOP_DEVICE ..."
mount "$LOOP_DEVICE" /var/lib/docker -o nodev
df
end_operation

echo "start docker ..."
start_operation "mount:startDocker"
# Start docker again
systemctl start docker
end_operation "end"

echo "check docker images ..."
docker images

# set environment variable of storage to be used to unmount.
export VSONLINE_STORAGE_SHARE=$CLOUDENVSTORAGE_SHARE
export VSONLINE_STORAGE_FILE=$CLOUDENVSTORAGE_FILE

# cleanup
unset SCRIPT_PARAM_STORAGE
unset CLOUDENVSTORAGE_ACCOUNT
unset CLOUDENVSTORAGE_PASS
unset CLOUDENVSTORAGE_SHARE
unset CLOUDENVSTORAGE_FILE
unset LOOP_DEVICE
echo "Done."
